/************************************************************

PhotoLine script library by Russell Cottrell.  Updated 6/8/2019.
Uses values specified in constants.js
Specify your browser at the bottom of this page.
Please visit russellcottrell.com/photo/PhotoLine/scripting.htm



function createImageDocument(pictureType, width, height, resolution, color)
Returns a new image document.
pictureType:  PTGray, PTRGB, PTCMYK, PT8Bit, PT16Bit, PT32Bit, PTAlpha
doc = createImageDocument(PTRGB+PT16Bit+PTAlpha, 900, 1500, 300, hexColor(Red, 0.5));

function createImageLayer(pictureType, left, top, width, height, color[, insert])
Returns a new image layer and optionally inserts it after the active layer.
pictureType as above.
Shortcut:  createImageLayer(pictureType, color[, insert]) creates a layer the same size as the document.
nLayer = createImageLayer(PTRGB, 100, 100, 200, 200, RGBColor(127, 127, 127), true);

function createTextLayer(text, fontName, fontSize, fontColor,
  left, top, width, height, paragraphAlignment[, insert])
Returns a new text layer and optionally inserts it after the active layer.
fontSize:  pixels
paragraphAlignment:  PALeft, PARight, PACenter, PAJustified, PAJustifiedAll
texLayer = createTextLayer("Hello", "Palatino Linotype", 48, hexColor(OliveDrab),
  0, 100, docSize[0], 48, PACenter, true);

function layerToRGB(bitDepth)
Converts the active layer to an RGB layer.
bitDepth:  8, 16, 32

function emboss(direction, height, width, intensity, curveShape, mode)
direction:  0-360
height:  5-90
width:  0-100
intensity:  0-100
curveShape:  CSStraight, CSStraightRaised, CSStraightLowered,
  CSReversed, CSReversedRaised, CSReversedLowered, CSArch, CSU
mode:  0 = Unsharp, 1 = Shade
pattern is Create From Intensity

function FloatToHex(num)
Converts a signed floating point number to a hex string for use in actions.

function ByteToHex(num)
Converts a byte (0 - 255) to a padded hex string.

function IntToHex(num)
Converts a signed integer (-32768 - 32767) to a padded hex string.

function LongToHex(num)
Converts a signed long (-2147483648 - 2147483647) to a padded hex string.

function PadString(str, char, n)
Pads a string with leading characters to length n.

function grayColor(gg[, aa])
Returns a grayscale color.  Alpha (opacity) is optional.
values:  gg 0-255; aa 0-1
color = grayColor(102, 0.5);

function RGBColor(rr, gg, bb[, aa])
Returns an RGB color.  Alpha is optional.
values:  rr, gg, bb 0-255; aa 0-1
color = RGBColor(187, 0, 0, 0.5);

function CMYKColor(cc, mm, yy, kk[, aa])
Returns a CMYK color.  Alpha is optional.
values:  cc, mm, yy, kk 0-100; aa 0-1
color = CMYKColor(3, 100, 63, 12);

function LabColor(L, a, b[, aa])
Returns an Lab color.  Alpha is optional.
values:  L 0-100; a, b -128-128; aa 0-1
color = LabColor(40, 65, 56);

function HISColor(hh, ii, ss[, aa])
Returns an HIS color.  Alpha is optional.
values:  hh 0-359; ii, ss 0-255; aa 0-1
color = HISColor(0, 37, 73);

function HSVColor(hh, ss, vv[, aa])
Returns an HSV color.  Alpha is optional.
values:  hh 0-359; ss, vv 0-255; aa 0-1
color = HSVColor(0, 100, 73);

function hexColor(hexStr[, aa])
Returns an RGB color.  Alpha is optional.
values:  hexStr "000000"-"ffffff"; aa 0-1
color = hexColor("bb0000");
color = hexColor(OSUScarlet, 0.5);

function transparent()
Returns a transparent color.
color = transparent();

function drawPolygon(pArray, color[, insert, merge])
Returns a new vector layer filled with a color.
  Optionally inserts it after the active layer and merges it with the active layer.
pArray is a list of coordinates that determine points:  x0, y0, x1, y1, etc.
Two points is a shortcut for the diagonals of a rectangle; more than two trace out a polygon.
drawPolygon([150,50, 55,119, 91,231, 209,231, 245,119], hexColor(Blue), true, false);

function drawEllipse(left, top, width, height, color[, half, insert, merge])
Returns a new vector layer filled with a color.
  Optionally inserts it after the active layer and merges it with the active layer.
left, top is the center point.
Use width = height for a circle.
half = true draws the upper half only.
drawEllipse(200, 200, 100, 100, hexColor(Gray), false, true, true);

function rotateLayer(angle, rx, ry)
Rotates the active layer.
angle is degrees counterclockwise.
rx, ry is the rotation reference point relative to the layer origin and dimensions;
  0, 0 is the origin (upper left); 0.5, 0.5 is the center point.

function getDocFolder(docPath)
Returns the folder of the specified document:  C:\document\folder

function getDocName(docPath)
Returns the name of the specified document:  image.jpg

function getScriptFolder()
Returns the PhotoLine script folder:  C:\Program Files\PhotoLine\Defaults[or PhotoLineSettings]\Automation

function getDesktop()
Returns the desktop folder:  C:\Users\User\Desktop

function readTextFile(filePath)
Returns the text of a text file, or an empty string if the file does not exist.

function writeTextFile(filePath, text)
Writes text to a text file, overwriting any existing text.  Creates the file if it does not exist.

function appendTextFile(filePath, text)
Appends a line to a text file.  Creates the file if it does not exist.

function fsoBuildPath(path, name)
Returns a name appended to a path, using the correct path separator.
Path can be absolute or relative and need not specify an existing folder.
databaseFile = fsoBuildPath(getDesktop(), "database.csv");

function runScript(file)
Runs a script file in the script folder.
runScript("script.js");

function openDialog(htmFile[, paramsArray])
Opens an .htm file in the script folder.
Specify the browser at the bottom of this page.
Optional paramsArray is appended as a query string:  file.htm?param1&param2
openDialog("file.htm", ["param1", "param2"]);

function alert(text)
Displays an alert box with the specified text.

************************************************************/



createImageDocument = function(pictureType, width, height, resolution, color) {

for (var i=1; i<=2; i++)
  arguments[i] = Math.round(arguments[i]);

var doc, image, Base64Str;

base64Str = "UGhvdG8gTGluZSBBY3Rpb25zAAAAAQAAAAABAP+bAAAADEdyb3VwQWN0aW9uAAIAAAAAAAAAAH4C" +
  "AQAAAAEAAAAEAAAAAQAAAAMAAABkAAAAAQD/mwAAAA1JQ0NOZXdBY3Rpb24AAQIAAAAAAAAAEgIB" +
  "AAAAAQAAAAQAAAAB/////wAAAAEAAAAEAAIAAAAAAAIAAAAGAQAAAAAAAAAAAwAAAAEAAAAABAAA" +
  "AAFA//////////8AAAABAAAAGP+bAAAAEnNldCBjb2xvciBwcm9maWxlAAAAAAIAAAAEAAAABf//" +
  "//8="; // set color profile

doc = new ActiveXObject("PhotoLine.Document");
doc.DocumentMode = false;
image = new ActiveXObject("PhotoLine.Image");
image.InitPicture(pictureType, [width, height], color);
doc.RootLayer.Insert(image, -1);
doc.Resolution = resolution;
doc.DoOperation("Action", "Data", base64Str, "ShowDialog", true);

return doc;

}


createImageLayer = function(pictureType, left, top, width, height, color, insert) {

var doc, aLayer, nLayer;

doc = pl.ActiveDocument;
aLayer = doc.ActiveLayer || doc.RootLayer.Last;

nLayer = new ActiveXObject("PhotoLine.Image");

if (arguments.length <= 3) {
  var color = arguments[1];
  var insert = (arguments.length <= 3 ? arguments[2] : false);
  var left = 0;
  var top = 0;
  var docSize = VBArray(doc.Size).toArray();
  var width = docSize[0];
  var height = docSize[1];
}

else {
  for (var i=1; i<=4; i++)
    arguments[i] = Math.round(arguments[i]);
}

nLayer.InitPicture(pictureType, [width, height], color);
nLayer.Origin = [left, top];

if (insert) {
  aLayer.Parent.Insert(nLayer, aLayer);
  doc.ActiveLayer = nLayer;
}

return nLayer;

}



createTextLayer = function(text, fontName, fontSize, fontColor,
  left, top, width, height, paragraphAlignment, insert) {

for (var i=4; i<=7; i++)
  arguments[i] = Math.round(arguments[i]);

var doc, aLayer, fontDict, paragraphDict, textLayer, range;

doc = pl.ActiveDocument;
aLayer = doc.ActiveLayer || doc.RootLayer.Last;

fontDict = new ActiveXObject("PhotoLine.Dictionary");
fontDict.Add("FamilyName", fontName, "Size", fontSize);

paragraphDict = new ActiveXObject("PhotoLine.Dictionary");
paragraphDict.Add("Alignment", paragraphAlignment);

textLayer = new ActiveXObject("PhotoLine.Text");
textLayer.Text = text;
range = [0, textLayer.TextLength];
textLayer.Origin = [left, top];
textLayer.Size = [width, height];
textLayer.SetAttribute(range, "Color", fontColor);
textLayer.SetAttribute(range, "Font", fontDict);
textLayer.SetAttribute(range, "Paragraph", paragraphDict);

if (insert) {
  aLayer.Parent.Insert(textLayer, aLayer);
  doc.ActiveLayer = textLayer;
}

return textLayer;

}



layerToRGB = function(bitDepth) {

var hexStr, doc;

switch(bitDepth) {
  case 8:
    bitDepth = "00";
    break;
  case 16:
    bitDepth = "20";
    break;
  case 32:
    bitDepth = "10";
    break;
}

hexStr = "50686f746f204c696e6520416374696f6e7300000001000000000100ff9b0000000e436f" +
  "6e76657274416374696f6e00020000000000000000120201000000010000000400000001" +
  "ffffffff000000010000000200010000000200000004" +
  bitDepth +
  "000000000000030000000400000003ffffffff";

doc = pl.ActiveDocument;
doc.DoOperation("Action", "Data", hexStr);

}



emboss = function(direction, height, width, intensity, curveShape, mode) {

var hexStr, doc;

direction = floatToHex(direction * ToRadians);

height = floatToHex(height * ToRadians);

width = byteToHex(width);

intensity = floatToHex(intensity);

mode = byteToHex(mode);

hexStr = "50686f746f204c696e6520416374696f6e7300000001000000000100ff9b0000000d456d" +
  "626f7373416374696f6e00010100000000000000120201000000010000000400000001ff" +
  "ffffff00000001000000e40001000000000000000000020000000000010000008c010000" +
  "0000050000000200" +
  mode +
  "0000000000000006" +
  direction +
  "0000000100000006" +
  height +
  "0000000200000006" +
  intensity +
  "0000000300000004000000" +
  width +
  "000000040000003e01000000000000000002000100000001000000260003" +
  curveShape +
  "ffffffffffffffff00000002000000360100000000000000000200030000000100000006" +
  "3243f6c0000000000002000000040000000300000003000000040000000affffffffffff" +
  "ffffffffffff";

doc = pl.ActiveDocument;
doc.DoOperation("Action", "Data", hexStr);

}



floatToHex = function(num) {

var hexStr, s, exp, expStr, len, man, manStr;

if (num == 0)
  hexStr = "000000000000";

else {

  s = (num < 0 ? -1 : 1);
  num = Math.abs(num);

  exp = Math.floor(Math.log(num) / Math.log(2)) + 1;
  expStr = exp;
  if (expStr < 0)
    expStr += 0x10000;
  expStr = expStr.toString(16);
  len = expStr.length;
  for (var i=0; i<4-len; i++)
    expStr = "0" + expStr;

  man = Math.floor(s * (num / Math.pow(2, exp)) * Math.pow(2, 30));
  if (s < 0)
    man += 0x100000000;

  manStr = man.toString(16);

  hexStr = manStr + expStr;

}

return hexStr;

}

byteToHex = function(num) {
return padString(Math.round(num).toString(16), "0", 2);
}

intToHex = function(num) {
return padString(Math.round(num).toString(16), "0", 4);
}

longToHex = function(num) {
return padString(Math.round(num).toString(16), "0", 8);
}

padString = function(str, char, n) {
var len = n - str.length;
for (var i=0; i<len; i++)
  str = char + str;
return str;
}



grayColor = function(gg, aa) {

gg /= 255;
var color = new ActiveXObject("PhotoLine.Color");
color.Model = CMGray;
if (aa !== undefined)
  color.Values = [gg, aa];
else
  color.Values = [gg];

return color;

}

RGBColor = function(rr, gg, bb, aa) {

rr /= 255;
gg /= 255;
bb /= 255;
var color = new ActiveXObject("PhotoLine.Color");
color.Model = CMRGB;
if (aa !== undefined)
  color.Values = [rr, gg, bb, aa];
else
  color.Values = [rr, gg, bb];

return color;

}

CMYKColor = function(cc, mm, yy, kk, aa) {

cc /= 100;
mm /= 100;
yy /= 100;
kk /= 100;
var color = new ActiveXObject("PhotoLine.Color");
color.Model = CMCMYK;
if (aa !== undefined)
  color.Values = [cc, mm, yy, kk, aa];
else
  color.Values = [cc, mm, yy, kk];

return color;

}

LabColor = function(L, a, b, aa) {

L /= 100;
a = (a + 128) / 256;
b = (b + 128) / 256;
var color = new ActiveXObject("PhotoLine.Color");
color.Model = CMLab;
if (aa !== undefined)
  color.Values = [L, a, b, aa];
else
  color.Values = [L, a, b];

return color;

}

HISColor = function(hh, ii, ss, aa) {

hh /= 359;
ii /= 255;
ss /= 255;
var color = new ActiveXObject("PhotoLine.Color");
color.Model = CMHIS;
if (aa !== undefined)
  color.Values = [hh, ii, ss, aa];
else
  color.Values = [hh, ii, ss];

return color;

}

HSVColor = function(hh, ss, vv, aa) {

hh /= 359;
ss /= 255;
vv /= 255;
var color = new ActiveXObject("PhotoLine.Color");
color.Model = CMHSV;
if (aa !== undefined)
  color.Values = [hh, ss, vv, aa];
else
  color.Values = [hh, ss, vv];

return color;

}

hexColor = function(hexStr, aa) {

var rr = parseInt(hexStr.substr(0, 2), 16)/255;
var gg = parseInt(hexStr.substr(2, 2), 16)/255;
var bb = parseInt(hexStr.substr(4, 2), 16)/255;

var color = new ActiveXObject("PhotoLine.Color");
color.Model = CMRGB;
if (aa !== undefined)
  color.Values = [rr, gg, bb, aa];
else
  color.Values = [rr, gg, bb];

return color;

}

transparent = function() {

var color = new ActiveXObject("PhotoLine.Color");
color.Model = CMGray;
color.Values = [0, 0];
return color;

}



drawPolygon = function(pArray, color, insert, merge) {

var doc, aLayer, vLayer, i;

doc = pl.ActiveDocument;
aLayer = doc.ActiveLayer || doc.RootLayer.Last;

vLayer = new ActiveXObject("PhotoLine.Vector");
vLayer.FillColor(0) = color;
vLayer.LineColor(0) = transparent();

if (pArray.length == 4) {

  vLayer.InsertPoints(0, VPTMoveTo,
    [pArray[0], pArray[1], pArray[2], pArray[1], pArray[2], pArray[3],
      pArray[0], pArray[3], pArray[0], pArray[1]]);

}

else {

  vLayer.InsertPoints(0, VPTMoveTo, [pArray[0], pArray[1]]);

  for (var i=2; i<pArray.length; i+=2)
    vLayer.InsertPoints(-1, VPTLineTo, [pArray[i], pArray[i+1]]);

  vLayer.InsertPoints(-1, VPTLineTo, [pArray[0], pArray[1]]);

}

if (insert) {
  aLayer.Parent.Insert(vLayer, aLayer);
  if (merge)
    doc.Merge([aLayer, vLayer]);
  else
    doc.ActiveLayer = vLayer;
}

if (!(insert && merge))
  return vLayer;

}



drawEllipse = function(left, top, width, height, color, half, insert, merge) {

var doc, aLayer, vLayer, i;
var x, y, a, b, controlLengthA, controlLengthB;
var points = [];

doc = pl.ActiveDocument;
aLayer = doc.ActiveLayer || doc.RootLayer.Last;

x = left;
y = top;
a = width/2;
b = height/2;
controlLengthA = a * 0.55191502449;
controlLengthB = b * 0.55191502449;

vLayer = new ActiveXObject("PhotoLine.Vector");
vLayer.FillColor(0) = color;
vLayer.LineColor(0) = transparent();

points[0] = x - a;
points[1] = y;
points[2] = points[0];
points[3] = y - controlLengthB;

points[4] = x - controlLengthA;
points[5] = y - b;
points[6] = x;
points[7] = points[5];
points[8] = x + controlLengthA;
points[9] = points[5];

points[10] = x + a;
points[11] = y - controlLengthB;
points[12] = points[10];
points[13] = y;
points[14] = points[10];
points[15] = y + controlLengthB;

points[16] = x + controlLengthA;
points[17] = y + b;
points[18] = x;
points[19] = points[17];
points[20] = x - controlLengthA;
points[21] = points[17];

points[22] = x - a;
points[23] = y + controlLengthB;
points[24] = points[0];
points[25] = y;

if (!half) {
  vLayer.InsertPoints(0, VPTMoveTo, [points[0], points[1]]);
  for (var i=2; i<points.length; i+=6)
    vLayer.InsertPoints(-1, VPTCurveTo,
      [points[i], points[i+1], points[i+2], points[i+3], points[i+4], points[i+5]]);
}
else {
  vLayer.InsertPoints(0, VPTMoveTo, [points[0], points[1]]);
  for (var i=2; i<14; i+=6)
    vLayer.InsertPoints(-1, VPTCurveTo,
      [points[i], points[i+1], points[i+2], points[i+3], points[i+4], points[i+5]]);
  vLayer.InsertPoints(-1, VPTLineTo, Array[points[0], points[1]]);
}

if (insert) {
  aLayer.Parent.Insert(vLayer, aLayer);
  if (merge)
    doc.Merge([aLayer, vLayer]);
  else
    doc.ActiveLayer = vLayer;
}

if (!(insert && merge))
  return vLayer;

}



rotateLayer = function(angle, rx, ry) {

var doc, aLayer, size, origin, referencePoint, matrix;

doc = pl.ActiveDocument;
aLayer = doc.ActiveLayer || doc.RootLayer.Last;

size = VBArray(aLayer.Size).toArray();
origin = VBArray(aLayer.Origin).toArray();
referencePoint = [origin[0] + size[0]*rx, origin[1] + size[1]*ry];

matrix = pl.GetRotationMatrix(angle, referencePoint);

aLayer.MatrixToPage = aLayer.MatrixToPage.Concatenate(matrix);

}



getDocFolder = function(docPath) {
return docPath.substr(0, docPath.lastIndexOf("\\")); // C:\document\folder
}

getDocName = function(docPath) {
return docPath.substr(docPath.lastIndexOf("\\") + 1); // image.jpg
}

getScriptFolder = function() {
var fso = new ActiveXObject("Scripting.FileSystemObject");
return fso.GetAbsolutePathName("."); // C:\Program Files\PhotoLine\Defaults\Automation
                                     // C:\Program Files\PhotoLine\PhotoLineSettings\Automation
}

getDesktop = function() {
var objShell = new ActiveXObject("WScript.Shell");
return objShell.SpecialFolders("Desktop"); // C:\Users\User\Desktop
}



readTextFile = function(filePath) {
var textFile, text;
text = "";
var fso = new ActiveXObject("Scripting.FileSystemObject");
if (fso.FileExists(filePath)) {
  textFile = fso.OpenTextFile(filePath, ForReading);
  if (!textFile.AtEndOfStream)
    text = textFile.ReadAll();
  textFile.Close();
}
return text;
}

writeTextFile = function(filePath, text) {
var textFile;
var fso = new ActiveXObject("Scripting.FileSystemObject");
textFile = fso.OpenTextFile(filePath, ForWriting, true);
textFile.Write(text);
textFile.Close();
}

appendTextFile = function(filePath, text) {
var textFile;
var fso = new ActiveXObject("Scripting.FileSystemObject");
textFile = fso.OpenTextFile(filePath, ForAppending, true);
textFile.WriteLine(text);
textFile.Close();
}



fsoBuildPath = function(path, name) {
var fso = new ActiveXObject("Scripting.FileSystemObject");
return fso.BuildPath(path, name);
}



runScript = function(file) {
var fso = new ActiveXObject("Scripting.FileSystemObject");
var scriptPath = fso.BuildPath(getScriptFolder(), file);
var scriptFile = fso.OpenTextFile(scriptPath, ForReading);
eval(scriptFile.ReadAll());
scriptFile.Close();
} 



openDialog = function(htmFile, paramsArray) {
var URL, objShell;
var fso = new ActiveXObject("Scripting.FileSystemObject");
URL = "file:///" + fso.BuildPath(getScriptFolder(), htmFile).replace(/\\/g, "/");
if (paramsArray)
  URL = URL + "?" + paramsArray.join("&");
URL = URL.replace(/ /g, "%20");
objShell = new ActiveXObject("Shell.Application");
objShell.ShellExecute(browser, URL, "", "open", 1);
}



alert = function(text) {
var objShell = new ActiveXObject("WScript.Shell");
objShell.Popup(text);
}



/********** Modify as needed: **********/


var fso = new ActiveXObject("Scripting.FileSystemObject");
with (fso)
  browser = BuildPath(GetParentFolderName(GetAbsolutePathName(".")), "QtWeb.exe");
  // C:\Program Files\PhotoLine\Defaults\QtWeb.exe
  // C:\Program Files\PhotoLine\PhotoLineSettings\QtWeb.exe

// browser = "C:\\Program Files\\Mozilla Firefox\\firefox.exe";
// browser = "C:\\Program Files (x86)\\Mozilla Firefox\\firefox.exe";






